///***************************************************************************************
// HTU21D Temperature/Humidity Sensor
// Do I2C Bit Banger
// Uses 1 MHz DCO for SIMCLK and MCLK
// Jed Margolin  5/21/2020.
//***************************************************************************************
#include <msp430.h>
#include <math.h>
#include "htu21.h"
#include <stdlib.h>
//================================================
int get_htu_temperature()
{
    unsigned char ack, data1, data2, crcdata;
//    unsigned long long temperature;
//----------------
// Start bit
    i2c_start_bit();

// send the ID
   ack = i2c_sendbyte(IIC_ID_WRITE);
//-------------
// send command
   ack |= i2c_sendbyte(HTU_trigger_temp);
   waitms(60);   // Datasheet says wait 50ms for measurement
//----------------
// Start bit
   i2c_start_bit();

// send the ID
   ack = i2c_sendbyte(IIC_ID_READ);
   waitms(1);   // Datasheet says
//----------------
   i2c_getbyte_ack();
   data1 = i2c_rxdata;

   i2c_getbyte_ack();
   data2 = i2c_rxdata;

   i2c_getbyte_nack();
   crcdata = i2c_rxdata;      // CRC

// Give it a Stop bit
   i2c_stop_bit();

/*
 * From the datasheet:  temperature = -46.85 + 175.72 * STemp/2^16
 *
 * Floating point is emulated. Although it produces a great deal of program code
 *   It does not take much longer to execute than the scaled integer math I tried
 *   to make work.
*/

   TempRaw = (data1 <<8) | (data2 & ~0x03);
//   TempRaw = 27112;  //  25.8 C
//   TempRaw = 20112;  //  7.0 C
//   TempRaw = 12000;  //  -14.6 C
//   TempRaw = 8000;  // -25.3 C; -13.5F

   fltemp = -46.85 + 175.72 * TempRaw/65536;

   savetemp = fltemp*10;

   ack = do_crc(data1, data2, crcdata);       // just for grins
//   ack = do_crc2(data1, data2, crcdata);       // just for grins

   asm(" nop");     // for debugger

   return(savetemp);
}

//==============================================
int get_htu_humidity()
{
    unsigned char ack, data1, data2, crcdata;
    unsigned long srh;
    long rht, temp1;
//----------------
// Start bit
    i2c_start_bit();

// send the ID
   ack = i2c_sendbyte(IIC_ID_WRITE);
//-------------
// send command
   ack |= i2c_sendbyte(HTU_trigger_hum);
   waitms(20);   // Datasheet says wait 16ms for measurement
//----------------
// Start bit
   i2c_start_bit();

// send the ID
   ack = i2c_sendbyte(IIC_ID_READ);
   waitms(1);
//----------------
   i2c_getbyte_ack();
   data1 = i2c_rxdata;

   i2c_getbyte_ack();
   data2 = i2c_rxdata;

   i2c_getbyte_nack();
   crcdata = i2c_rxdata;      // CRC

// Give it a Stop bit
   i2c_stop_bit();

/*
 * The formula in the manual for humidity correction is
 * RH = -6 + 125 * SRH / 2^16, which is
 * RH = (SRH * 125 / 65536) -6
 *
*/
   srh = (data1 <<8) | (data2 & ~0x03);  // the 2 LSB are status and must be zeroed out
   srh = (srh * 125) >>16;
   srh = srh - 6;

/*
   * Now correct for temperature since the data for the sensor is for 25 degrees C
   * From the manual: RH(T) = RH(Measured) + ((25-T) * -0.15)
   * Again, scale it so we can use integers
   * Divides are brutal so we will use shifts that are 99.93% accurate
   * RH(T) = ( RH(Measured)*2048 + (25-raw_temp) * (-0.15 * 2048) ) /2048
   * RH(T) = { RH(Measured)* 2048 + (25-raw_temp) * -307 } /2048
*/
   temp1 = (25 - savetemp/100);
   temp1 = temp1 * (-307);
   rht = ( (srh <<11) + temp1);
   rht = rht>>11;

   do_crc(data1, data2, crcdata);       // just for grins

   asm(" nop");     // for debugger

   return(rht);
}
//=====================================
void i2c_start_bit()

{

// Make sure SCL is low since an SDA transition with SCL low will be ignored
   SCL_POUT &= ~SCL_BIT;

// Make SDA high
   SDA_POUT |= SDA_BIT;
   wait(7);

   SCL_POUT |= SCL_BIT;   // SCL high
   wait(5);
   SDA_POUT &= ~SDA_BIT;   //SDA Low
   wait(2);

   SCL_POUT &= ~SCL_BIT;   //SCL low
   wait(2);
}
//===================================================
void i2c_stop_bit()
{

// Make sure SCL is low since an SDA transition with SCL low will be ignored
   SCL_POUT &= ~SCL_BIT;
   wait(4);

// Make SDA low
   SDA_POUT &= ~SDA_BIT;
   wait(8);

// Make SCL high
   SCL_POUT |= SCL_BIT;   // SCL high

   wait(8);
   SDA_POUT |= SDA_BIT;   //SDA high

   wait(10);
}
//=======================================
unsigned char i2c_getbyte_nack()
{
    unsigned char n, data;


// Make SDA an input
   SDA_PDIR &= ~SDA_BIT;   // In

   data = 0;

   for(n=8; n>0; n--)
   {
      data = data<<1;
      if(SDA_PIN & SDA_BIT)
      {
         data = data | 0x01;   // SDA high
      }

      wait(6);
      SCL_POUT |= SCL_BIT;   //SCL high
      wait(7);
      SCL_POUT &= ~SCL_BIT;   //SCL low
   }

//       wait(7);

// We are responsible for sending ACK or NACK
// so make SDA an output and make it high (NACK)
    SDA_PDIR |= SDA_BIT;     //Out/
    SDA_POUT |= SDA_BIT;

    wait(4);
    SCL_POUT |= SCL_BIT;   //SCL high
    wait(10);
    SCL_POUT &= ~SCL_BIT;   //SCL low

    SDA_POUT |= SDA_BIT;  // SDA back to high to get ready for the stop bit

// and we are done
    i2c_rxdata = data;
    return(0);
}
//=======================================
unsigned char i2c_getbyte_ack()
{
   unsigned char n, data;

// Make SDA an input
   SDA_PDIR &= ~SDA_BIT;   // In

   data = 0;

   for(n=8; n>0; n--)
   {
      data = data<<1;
      if(SDA_PIN & SDA_BIT)
      {
         data = data | 0x01;   // SDA high
      }

      wait(6);
      SCL_POUT |= SCL_BIT;   //SCL high
      wait(7);
      SCL_POUT &= ~SCL_BIT;   //SCL low
   }

// We are responsible for sending ACK
// so make SDA an output and make it low (NACK)
   SDA_PDIR |= SDA_BIT;     //Out/
   SDA_POUT &= ~SDA_BIT;

   wait(4);
   SCL_POUT |= SCL_BIT;   //SCL high
   wait(10);
   SCL_POUT &= ~SCL_BIT;   //SCL low

   SDA_POUT |= SDA_BIT;  // SDA back to high to get ready for the stop bit

// and we are done
   i2c_rxdata = data;
   return(0);
}
//==================================
void wait(int i)
{
   unsigned int n;

   for(n=i; n>0; n--)
   {
     asm(" nop");
   }
}
//=======================================================
void htu_init(void)
{
   unsigned char ack;

// Start bit
   i2c_start_bit();

// send the ID
   ack = i2c_sendbyte(IIC_ID_WRITE);

// send command
   ack |= i2c_sendbyte(HTU_soft_reset);
//   wait114ms();
   waitms(100);

// Give it a Stop bit
   i2c_stop_bit();

   asm(" nop");     // for debugger
}
//==========================================================
void wait1()
{
volatile unsigned int i;

    i = 0;
//    for(i=0; i<1; i++){}
    return;
}
//=================================================================
// Write to a register with i2c
unsigned char i2c_write(unsigned char address, unsigned char data)
{
    unsigned char ack;

// Start bit
    i2c_start_bit();

// send the ID
    ack = i2c_sendbyte(IIC_ID_WRITE);

// we will keep going even if ack is high
    ack = ack | i2c_sendbyte(address);

// Send the data
//    ack = ack | i2c_sendbyte(data);

 // Give it a Stop bit
    i2c_stop_bit();

    return(ack);
 }
//==========================================================
// Read i2C byte
unsigned char i2c_read(unsigned char address)
{
    unsigned char ack;

// Start bit
    i2c_start_bit();

// send the ID
   ack = i2c_sendbyte(IIC_ID_WRITE);

// we will keep going even if ack is high
   ack = ack | i2c_sendbyte(address);

// How does the slave send the data back?

// Repeat the start bit
   i2c_start_bit();

// Send the device ID
   ack = ack | i2c_sendbyte(IIC_ID_READ);

   ack = ack | i2c_getbyte();     // data ic2_rxdata is global

// Give it a Stop bit
   i2c_stop_bit();

   return(i2c_rxdata);      // global
}
//=================================================
unsigned char i2c_sendbyte(unsigned char data)
{
    unsigned char n, ack;

    for(n=8; n>0; n--)
    {
      if(data & 0x80)
      {
         SDA_POUT |= SDA_BIT;   // SDA high
      }
      else {SDA_POUT &= ~SDA_BIT;}  // SDA low
      data = data<<1;

      wait(6);
      SCL_POUT |= SCL_BIT;   //SCL high
      wait(7);
      SCL_POUT &= ~SCL_BIT;   //SCL low
//       wait(10);
    }

// look for ACK from I2C device
    SDA_PDIR &= ~SDA_BIT;   // Input

    wait(4);
    SCL_POUT |= SCL_BIT;   //SCL high


// ack is immediately available
    if(SDA_PIN & SDA_BIT)
    {
       ack = 1;
    }
    else {ack = 0;}

    wait(8);

// Then make SCL low
    SCL_POUT &= ~SCL_BIT;   //SCL low

// Give the slave time to release SDA
    wait(100);

// Then make SDA an output again and low; SCL is low so transitions won't matter
    SDA_POUT &= ~SDA_BIT;
    SDA_PDIR |= SDA_BIT;     // An output again

    wait(10);

    return(ack);
}
//=======================================
unsigned char i2c_getbyte()
{
   unsigned char n, data;

// Make SDA an input
   SDA_PDIR &= ~SDA_BIT;   // In

   data = 0;

   for(n=8; n>0; n--)
   {
      data = data<<1;
      if(SDA_PIN & SDA_BIT)
      {
         data = data | 0x01;   // SDA high
      }

      wait(6);
      SCL_POUT |= SCL_BIT;   //SCL high
      wait(7);
      SCL_POUT &= ~SCL_BIT;   //SCL low
   }

// We are responsible for sending ACK or NACK
// so make SDA an output
   SDA_PDIR |= SDA_BIT;     //Out/

// An ACK tells the slave to keep sending data, a NACK says no more
// We are only asking for one byte here so send a NACK
   SDA_POUT |= SDA_BIT;

   wait(4);
   SCL_POUT |= SCL_BIT;   //SCL high
   wait(10);
   SCL_POUT &= ~SCL_BIT;   //SCL low

   SDA_POUT |= SDA_BIT;  // SDA back to high to get ready for the stop bit

// and we are done
   i2c_rxdata = data;
   return(0);
}
//====================================
void do_register()
{
    unsigned char ack, data, reserved;

    data = BIT1;  // disable OTP reload  // BIT7 = BIT6 = BIT2 = = 0
//---------------------
// Start bit
   i2c_start_bit();

// send the ID
   ack = i2c_sendbyte(IIC_ID_WRITE);

// send the read-register command
   ack = i2c_sendbyte(HTU_read_register);   // should be ack = 0
//---------------------
// Start bit
   i2c_start_bit();

   ack = i2c_sendbyte(IIC_ID_READ);

   i2c_getbyte_nack();  // in i2c_rxdata
   data = i2c_rxdata;
//-----------------------
//  Now write it with the reserve bits unchanged d3, d4, d5
   reserved = data & 0x38;
   data = (data & ~(0x38)) | reserved | BIT1;

// Start bit
   i2c_start_bit();

// send the ID
   ack = i2c_sendbyte(IIC_ID_WRITE);
//--------------------
// send the write register command
   ack = i2c_sendbyte(HTU_write_register);   // should be ack = 0

// now the data
   ack = i2c_sendbyte(data);

// Stop bit
   i2c_stop_bit();
//-----------------------
}
//===========================================
int do_crc(unsigned char data1, unsigned char data2, unsigned char crc)
{
    unsigned long int datareg, divisor, mask;
    unsigned int n;

//  from datasheet: datareg = 0x4E85; crc = 0x6B
//    data1 = 0x4E;
//    data2 = 0x85;     // datareg = 0x4E8500;
//    crc = 0x6B;

    datareg =(data1<<8 | data2);     // Code Composer Studio 9 needs it done this way
    datareg = datareg<<8;            //    or it will not compile it correctly

//   divisor = 0x131;    //  x^8 + x^5 + x^4 + 1
//   divisor = divisor<<15
    divisor = 0x988000;      // 0x131<<15
    mask = 0x800000;
    for(n=16; n>0; n--)
    {
       if(datareg & mask)
       {
         datareg = datareg ^ divisor;
       }
       divisor = divisor>>1;
       mask = mask>>1;
    }

    asm(" nop");   // for debugger

//    return(datareg-crc);    // '0' means CRC check ok
    if( datareg == crc) return(0);   //CRC Check OK
    return(1);  // CRC Check failed
}
//===========================================
int do_crc2(unsigned char data1, unsigned char data2, unsigned char crc)
{
    unsigned long int datareg, divisor, quotient, remainder;

//  from datasheet: datareg = 0x4E85; crc = 0x6B
    data1 = 0x4E;
    data2 = 0x85;     // datareg = 0x4E8500;
    crc = 0x6B;

    datareg =(data1<<8 | data2);     // Code Composer Studio 9 needs it done this way
    datareg = datareg<<16;            //    or it will not compile it correctly

    divisor = 0x9880000;
    quotient = datareg/(divisor);
    remainder = datareg % (divisor);
    remainder = remainder<<1;
    remainder = remainder<<1;
    remainder = remainder<<1;
    remainder = remainder<<1;

    quotient = datareg*crc;
    quotient = quotient >>1;
    quotient = quotient >>1;
    quotient = quotient >>1;

    asm(" nop");   // for debugger

    return(1);  // CRC Check failed
}
//==========================================
