//*********************************************************************************
// HTU21D Temperature/Humidity Sensor
// Do I2C manually
// Uses 1 MHz DCO for SIMCLK and MCLK
// Jed Margolin  5/17/2019.
//*********************************************************************************
// I2C Interface
// These need to be initialized in Main
#define SCL_POUT  P1OUT
#define SCL_BIT   BIT5

#define SDA_POUT  P1OUT
#define SDA_PIN   P1IN
#define SDA_PDIR  P1DIR
#define SDA_BIT   BIT7
//=======================================
#define IIC_ID_WRITE 0x80    // (0x40<<1)        HTU21 device
#define IIC_ID_READ 0x81     // (0x40<<1) & 0x01

#define HTU_trigger_temp   0xF3
#define HTU_trigger_hum    0xF5
#define HTU_write_register 0xE6
#define HTU_read_register  0xE7
#define HTU_soft_reset     0xFE
#define HTU_SNAC1          0xFC
#define HTU_SNAC2          0xC9
#define HTU_SNB1           0xFA
#define HTU_SNB2           0x0F

/*
 sn is 64 bits
 sna 16 bits  0x4854
 snb  32 bits
 snc  16 bits
*/

#define hum_lsb 0xFE;       // humidity byte 0      Read Only  bmpdata[7]
#define hum_msb 0xFD;       // humidity byte 1      Read Only  bmpdata[6]
#define temp_xlsb 0xFC;     // temperature byte 0   Read Only  bmpdata[5]   R,Address,Data
#define temp_lsb 0xFB;      // temperature byte 1   Read Only  bmpdata[4]
#define temp_msb 0xFA;      // temperature byte 2   Read Only  bmpdata[3]

#define bme_config 0xF5;    // configuration        Read/Write  R/W,Address;Data
#define bme_ctrl_meas 0xF4; // control measurement  Read/Write  R/W,Address;Data
#define bme_status 0xF3;    // status               Read Only
#define bme_reset 0xE0;     // reset                Write Only  /W,Address;Data
#define bme_id 0xD0;        // part id = 0x58       Read Only

unsigned char i2c_read(unsigned char);
void i2c_stop_bit(void);
void i2c_start_bit(void);
unsigned char i2c_sendbyte(unsigned char);
unsigned char i2c_getbyte_ack(void);
unsigned char i2c_getbyte_nack(void);
unsigned char i2c_getbyte();

void do_register(void);
void test_htu(void);
void test2_htu(void);

int get_htu_temperature(void);
int get_htu_humidity(void);
int do_crc(unsigned char, unsigned char, unsigned char);

unsigned char i2c_write(unsigned char, unsigned char);

void htu_init(void);
int wait_ready(void);

void wait(int);
void wait1(void);

//void lcd_wr_data(unsigned int);
void lcd_wr_bcd(unsigned int, unsigned int);
void lcd_wr_bcd6(unsigned int, signed long);
int test_bcd(void);

// Globals
unsigned char i2c_rxdata;
unsigned char intflag, intcount;

unsigned char bmedata[8];
unsigned char humdata[8];

int temp_C, temp_F;
long T, F, hextemp;
unsigned int Humidity;
void waitms(unsigned int);
long TempRaw;
signed int savetemp;
float fltemp;
//================================================
