/*
 For ht5u21d display
 Jed Margolin  5/21/2020
*/
//=============================================
#include <msp430.h>
#include "lcd.h"
#include "htu21.h"

void lcd_init(void)
{
    lcd_reset();
    domsg(&msg1);    // "htu-21d test"
    domsg(&msg2);    // "Jed Margolin"
}
//=========================
void lcd_reset(void)
{
/*
P2.0 - LCD D4
P2.1 - LCD D5
P2.2 - LCD D6
P2.3 - LCD D7

P2.4 - LCD E (Enable High, Write Data, Enable Low)
P2.5 - LCD RS (Register Select; RS low = Instruction, RS High = Data)
 */

// Wait at least 16 ms after power-up
    wait10ms();    // wait 10 ms
    wait10ms();    // wait 10 ms

    P2OUT &= (~BIT5);    // RS = 0

    lcd_wr4(0x03);      // write nybble and wait 200us
    wait10ms();

    lcd_wr4(0x03);      // write nybble and wait 200us
    lcd_wr4(0x03);      // write nybble and wait 200us
    lcd_wr4(0x02);      // write nybble and wait 200us
    lcd_wr_instr(0x28); //  2 lines, 5x7 dots
    lcd_wr_instr(0x08); //  display off
    lcd_wr_instr(0x01); //  display on

    wait10ms();

//    lcd_wr_instr(0x0F); //  display on, cursor on, blink on
    lcd_wr_instr(0x0c);  //  display on, cursor oFF, blink oFF

    lcd_wr_instr(0x06);  //  entry mode set: cursor increment, no shift
}
//==================================
// wait 10 ms
void wait10ms(void)
{
   wait_lcd(41);			
}
//==================================
void wait_lcd(unsigned int time)        // 500 uS
{
   volatile unsigned int i,j;

   for(i=time; i>0; i--)
   {
      for(j=24; j>0; j--);		// was 382 for 16MHz clock
      {
      }
   }

}
//==========================================================
void lcd_wr4(unsigned int data)      // write nybble and wait 200us
{

//    P2.4 - LCD E (Enable High, Write Data, Enable Low)
//    P2.5 - LCD RS (Register Select; RS low = Instruction, RS High = Data)

    P2OUT |= BIT4;     // Enable High

    data &= 0x0F;
    P2OUT &= 0xF0;
    P2OUT |= data;     // write nybble

    P2OUT &= (~BIT4);  //  /E low

    wait_lcd(20);         // 200 us
}
//======================
// LCD - Write 8-bit instruction
// P2.5 - LCD RS (Register Select; RS low = Instruction, RS High = Data)
void lcd_wr_instr(unsigned int data)
{
   unsigned int dsave;

   P2OUT &= (~BIT5);    // RS = 0

   dsave = data;
   dsave  = dsave >>4;       // high nybble
   lcd_wr4(dsave);  // write nybble and wait 200 us

   data &=0xF;      // low nybble4
   lcd_wr4(data);
}
//======================
// LCD - Write 8-bit data
// P2.5 - LCD RS (Register Select; RS low = Instruction, RS High = Data)
void lcd_wr_data(unsigned int data)
{
    unsigned int dsave;

    P2OUT |= BIT5;    // RS = 1

    dsave = data;
    dsave = dsave >>4;       // high nybble
    lcd_wr4(dsave);  // write nybble and wait 200 us

    data &=0xF;      // low nybble4
    lcd_wr4(data);
}
//======================
void lcd_wr_bcd(unsigned int pos, unsigned int bin)
{
// First Convert Binary to ASCII
// Adapted from 8-bit microcontroller code with a DAA instruction

// unsigned int bin = 0x386; // 0x67 = 103d; 0x386 = 902d
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
   unsigned char char1, char2, char3;
   int i;
   unsigned char hcarry;

   for(i=0; i<12; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x800)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

     if(bcd2 >9)
     {
         bcd2 = bcd2 - 10;
//       bcd3 = bcd3 + 1;
//       hcarry = 1;
     }
//        else {hcarry = 0;}

   }

// Convert to ascii
   char1 = bcd2 + 0x30;
   char2 = bcd1 + 0x30;
   char3 = bcd0 + 0x30;

// Display it
   lcd_wr_instr(pos);

   if(char1 == 0x01)
      {lcd_wr_data(char1);}
   else {lcd_wr_data(' ');}

   lcd_wr_data(char2);
   lcd_wr_data(char3);

}
//=========================
// clear a line
void lcd_clear(unsigned char address)
{
   int n;

   lcd_wr_instr(address);

   for(n=1; n<16; n++)
   {
       lcd_wr_data(' ');
   }
}
//=========================
void lcd_wr_hex(unsigned char pos, unsigned char hex)
{
    unsigned char digit1, digit2;

    digit1 = hex>>4;
    digit2 = hex & 0xF;

// Convert to ascii
   if(digit1<10)  {digit1 = digit1 + 0x30;}
   else {digit1 = digit1 + 0x37;}

   if(digit2<10)  {digit2 = digit2 + 0x30;}
   else {digit2 = digit2 + 0x37;}

   // Display it
   lcd_wr_instr(pos);
   lcd_wr_data(digit1);
   lcd_wr_data(digit2);
}
//=====================================================
void lcd_wr_bcd6(unsigned int pos, long bin)
{
    unsigned char negative = 0;

    if(bin<0)
    {
       bin = -bin;
       negative = 1;
    }

// First Convert Binary to ASCII
// Adapted from 8-bit microcontroller code with a DAA instruction
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
   unsigned char bcd3 = 0;
   unsigned char bcd4 = 0;
   unsigned char bcd5 = 0;
   unsigned char bcd6 = 0;
// unsigned char bcd7 = 0;

   unsigned char char1, char2, char3, char4, char5, char6;
// unsigned char char7;

   int i;
   unsigned char hcarry;

//  signed long bin1 = 1048575;             // maximun value is 1048575 = 0xFFFFF
//  bin = bin1;

   for(i=0; i<20; i++)      // 16
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x80000)      // 0x8000
      {bcd0 = bcd0 +1;}

//      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

      if(bcd2 >9)
      {
         bcd2 = bcd2 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD3
     bcd3 = bcd3<<1;
     bcd3 = bcd3 + hcarry;

     if(bcd3 >9)
     {
        bcd3 = bcd3 - 10;
        hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD4
     bcd4 = bcd4<<1;
     bcd4 = bcd4 + hcarry;

     if(bcd4 >9)
     {
        bcd4 = bcd4 - 10;
        hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD5
     bcd5 = bcd5<<1;
     bcd5 = bcd5 + hcarry;

     if(bcd5 >9)
     {
        bcd5 = bcd5 - 10;
        hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD6
     bcd6 = bcd6<<1;
     bcd6 = bcd6 + hcarry;

     if(bcd6 >9)
     {
        bcd6 = bcd6 - 10;
        hcarry = 1;
//      bcd7 = 1;
     }
     else {hcarry = 0;}

     bin = bin<<1;
   }

// Convert to ascii
   char1 = bcd6 + 0x30;
   char2 = bcd5 + 0x30;
   char3 = bcd4 + 0x30;
   char4 = bcd3 + 0x30;
   char5 = bcd2 + 0x30;
   char6 = bcd1 + 0x30;
// char7 = bcd0 + 0x30;

// Remove leading zeros
   for(i=4; i>0; i++)
   {
      if(char1 != 0x30) {break;}
      else {char1 = ' ';}

      if(char2 != 0x30) {break;}
      else {char2 = ' ';}

      if(char3 != 0x30) {break;}
      else {char3 = ' ';}

      if(char4 != 0x30) {break;}
      else {char4 = ' ';}
   }

// Display it
   lcd_wr_instr(pos);
   if(negative)
      {lcd_wr_data('-');}

//   lcd_wr_data(char1);
   if(char2 != ' ')
       {lcd_wr_data(char2);}

   if(char3 != 0x20)
       {lcd_wr_data(char3);}

   if(char4 != 0x20)
       {lcd_wr_data(char4);}

   if(char5 != 0x20)
       {lcd_wr_data(char5);}

   if(char6 != 0x20)
       {lcd_wr_data(char6);}

}
//============================================
void lcd_wr_bcd7(unsigned int pos, long bin)
{
    unsigned char negative = 0;

    if(bin<0)
    {
       bin = -bin;
       negative = 1;
    }

// First Convert Binary to ASCII
// Adapted from 8-bit microcontroller code with a DAA instruction

   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
   unsigned char bcd3 = 0;
   unsigned char bcd4 = 0;
   unsigned char bcd5 = 0;
   unsigned char bcd6 = 0;
// unsigned char bcd7 = 0;

   unsigned char char1, char2, char3, char4, char5, char6, char7;
   int i;
   unsigned char hcarry;

//  signed long bin1 = 1048575;             // maximun value is 1048575 = 0xFFFFF
//  bin = bin1;

   for(i=0; i<20; i++)      // 16
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x80000)      // 0x8000
      {bcd0 = bcd0 +1;}

//      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

      if(bcd2 >9)
      {
         bcd2 = bcd2 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD3
     bcd3 = bcd3<<1;
     bcd3 = bcd3 + hcarry;

     if(bcd3 >9)
     {
        bcd3 = bcd3 - 10;
        hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD4
     bcd4 = bcd4<<1;
     bcd4 = bcd4 + hcarry;

     if(bcd4 >9)
     {
        bcd4 = bcd4 - 10;
        hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD5
     bcd5 = bcd5<<1;
     bcd5 = bcd5 + hcarry;

     if(bcd5 >9)
     {
        bcd5 = bcd5 - 10;
        hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD6
     bcd6 = bcd6<<1;
     bcd6 = bcd6 + hcarry;

     if(bcd6 >9)
     {
        bcd6 = bcd6 - 10;
        hcarry = 1;
//      bcd7 = 1;
     }
     else {hcarry = 0;}

     bin = bin<<1;
   }

// Convert to ascii
   char1 = bcd6 + 0x30;
   char2 = bcd5 + 0x30;
   char3 = bcd4 + 0x30;
   char4 = bcd3 + 0x30;
   char5 = bcd2 + 0x30;
   char6 = bcd1 + 0x30;
   char7 = bcd0 + 0x30;


// Remove leading zeros
   for(i=4; i>0; i++)
   {
      if(char1 != 0x30) {break;}
      else {char1 = ' ';}

      if(char2 != 0x30) {break;}
      else {char2 = ' ';}

      if(char3 != 0x30) {break;}
      else {char3 = ' ';}

      if(char4 != 0x30) {break;}
      else {char4 = ' ';}
   }

// Display it
   lcd_wr_instr(pos);
   if(negative)
      {lcd_wr_data('-');}

//   lcd_wr_data(char1);
   if(char2 != ' ')
       {lcd_wr_data(char2);}

   if(char3 != 0x20)
       {lcd_wr_data(char3);}

   if(char4 != 0x20)
       {lcd_wr_data(char4);}

   if(char5 != 0x20)
       {lcd_wr_data(char5);}

   if(char6 != 0x20)
       {lcd_wr_data(char6);}

   lcd_wr_data('.');
   lcd_wr_data(char7);

}
//============================================
void lcd_display(void)
{
    lcd_clear(0x80);		// clear the line starting at 0x80 (Line 1)
    lcd_clear(0xC0);		// clear the line starting at 0xC0 (Line 2)

    lcd_wr_bcd6(0x80,TempRaw*10);   // Raw temperature value

    lcd_wr_bcd7(0x88, savetemp);   // Display Temperature at address
    lcd_wr_data(' ');       // ASCII
    lcd_wr_data('C');       // ASCII

    lcd_wr_bcd7(0xC0, F);   // Display Temperature at address
    lcd_wr_data(' ');       // ASCII
    lcd_wr_data('F');       // ASCII

    lcd_wr_bcd(0xC9, Humidity);    // Humidity
    lcd_wr_instr(0xCD);
    lcd_wr_data('%');
}
//===============================================================
// Do LCD Message
void domsg(const struct messages *sptr)
{
    unsigned char m,n;
    unsigned char data;

    m = sptr->pos;  // position
    lcd_wr_instr(m);

    m = sptr->numchars;  // numbers of chars

    for(n=0; n<m; n++)
    {
        data = sptr->msg[n];
        lcd_wr_data(data);
    }
}
//===============================================================
